import { isString } from "lodash-es";
class InlineChunkHtmlPlugin {
  htmlWebpackPlugin;
  tests;
  constructor(htmlWebpackPlugin, tests) {
    this.htmlWebpackPlugin = htmlWebpackPlugin;
    this.tests = tests;
  }
  /**
   * Get the inlined tag based on the specified conditions.
   * @param publicPath - The public path used in the project.
   * @param assets - Compilation assets.
   * @param tag - The HTML tag to be processed.
   * @returns The modified HTML tag.
   */
  getInlinedTag(publicPath, assets, tag) {
    if (tag.tagName !== "script") {
      return tag;
    }
    if (!tag?.attributes?.src || !isString(tag?.attributes?.src)) {
      return tag;
    }
    const scriptName = publicPath ? tag.attributes.src.replace(publicPath, "") : tag.attributes.src;
    if (!this.tests.some((test) => scriptName.match(test))) {
      return tag;
    }
    const asset = assets[scriptName];
    if (asset == null) {
      return tag;
    }
    return {
      tagName: "script",
      innerHTML: asset.source(),
      attributes: {},
      voidTag: false,
      meta: {}
    };
  }
  /**
   * Apply the plugin to the Webpack compiler.
   * @param compiler - The Webpack compiler instance.
   */
  apply(compiler) {
    let publicPath = compiler.options.output.publicPath || "";
    if (publicPath && !publicPath.endsWith("/")) {
      publicPath += "/";
    }
    compiler.hooks.compilation.tap(
      "InlineChunkHtmlPlugin",
      (compilation) => {
        const tagFunction = (tag) => this.getInlinedTag(publicPath, compilation.assets, tag);
        const hooks = this.htmlWebpackPlugin.getHooks(compilation);
        hooks.alterAssetTagGroups.tap("InlineChunkHtmlPlugin", (assets) => {
          assets.headTags = assets.headTags.map(tagFunction);
          assets.bodyTags = assets.bodyTags.map(tagFunction);
          return assets;
        });
        hooks.afterEmit.tap("InlineChunkHtmlPlugin", (assets) => {
          Object.keys(compilation.assets).forEach((assetName) => {
            if (this.tests.some((test) => assetName.match(test))) {
              delete compilation.assets[assetName];
            }
          });
          return assets;
        });
      }
    );
  }
}
var InlineChunkHtmlPlugin_default = InlineChunkHtmlPlugin;
export {
  InlineChunkHtmlPlugin_default as default
};
