import path from "path";
import pico from "picocolors";
class ModuleNotFoundPlugin {
  appPath;
  constructor(appPath) {
    this.appPath = appPath;
    this.getRelativePath = this.getRelativePath.bind(this);
    this.prettierError = this.prettierError.bind(this);
  }
  getRelativePath(_file) {
    let file = path.relative(this.appPath, _file);
    if (file.startsWith("..")) {
      file = _file;
    } else if (!file.startsWith(".")) {
      file = "." + path.sep + file;
    }
    return file;
  }
  prettierError(err) {
    const { details: _details = "", origin } = err;
    if (origin == null) {
      const caseSensitivity = err.message && /\[CaseSensitivePathsPlugin\] `(.*?)` .* `(.*?)`/.exec(err.message);
      if (caseSensitivity) {
        const [, incorrectPath, actualName] = caseSensitivity;
        const actualFile = this.getRelativePath(
          path.join(path.dirname(incorrectPath), actualName)
        );
        const incorrectName = path.basename(incorrectPath);
        err.message = `Cannot find file: '${incorrectName}' does not match the corresponding name on disk: '${actualFile}'.`;
      }
      return err;
    }
    const file = this.getRelativePath(origin.resource);
    let details = _details.split("\n");
    const request = /resolve '(.*?)' in '(.*?)'/.exec(details);
    if (request) {
      const isModule = details[1] && details[1].includes("module");
      const isFile = details[1] && details[1].includes("file");
      const [, target, context] = request;
      if (isModule) {
        details = [
          `Cannot find module: '${target}'. Make sure this package is installed.`,
          "",
          "You can install this package by running: " + pico.bold(`pnpm install ${target}`) + "."
        ];
      } else if (isFile) {
        details = [
          `Cannot find file '${target}' in '${this.getRelativePath(context)}'.`
        ];
      } else {
        details = [err.message];
      }
    } else {
      details = [err.message];
    }
    err.message = [file, ...details].join("\n").replace("Error: ", "");
    const isModuleScopePluginError = err.error && err.error.__module_scope_plugin;
    if (isModuleScopePluginError) {
      err.message = err.message.replace("Module not found: ", "");
    }
    return err;
  }
  apply(compiler) {
    const { prettierError } = this;
    compiler.hooks.make.intercept({
      register(tap) {
        if (!(tap.name === "MultiEntryPlugin" || tap.name === "EntryPlugin")) {
          return tap;
        }
        return Object.assign({}, tap, {
          fn: (compilation, callback) => {
            tap.fn(compilation, (err, ...args) => {
              if (err && err.name === "ModuleNotFoundError") {
                err = prettierError(err);
              }
              callback(err, ...args);
            });
          }
        });
      }
    });
    compiler.hooks.normalModuleFactory.tap("ModuleNotFoundPlugin", (nmf) => {
      nmf.hooks.afterResolve.intercept({
        register(tap) {
          if (tap.name !== "CaseSensitivePathsPlugin") {
            return tap;
          }
          return Object.assign({}, tap, {
            fn: (compilation, callback) => {
              tap.fn(compilation, (err, ...args) => {
                if (err && err.message && err.message.includes("CaseSensitivePathsPlugin")) {
                  err = prettierError(err);
                }
                callback(err, ...args);
              });
            }
          });
        }
      });
    });
  }
}
var ModuleNotFoundPlugin_default = ModuleNotFoundPlugin;
export {
  ModuleNotFoundPlugin_default as default
};
