import path from "path";
import fs from "fs-extra";
import webpack from "webpack";
import resolve from "resolve";
import HtmlWebpackPlugin from "html-webpack-plugin";
import MiniCssExtractPlugin from "mini-css-extract-plugin";
import { WebpackManifestPlugin } from "webpack-manifest-plugin";
import { BundleAnalyzerPlugin } from "webpack-bundle-analyzer";
import WorkboxWebpackPlugin from "workbox-webpack-plugin";
import ESLintPlugin from "eslint-webpack-plugin";
import StylelintPlugin from "stylelint-webpack-plugin";
import CaseSensitivePathsPlugin from "case-sensitive-paths-webpack-plugin";
import ReactRefreshWebpackPlugin from "@pmmmwh/react-refresh-webpack-plugin";
import WebpackBar from "webpackbar";
import ForkTsCheckerWebpackPlugin from "fork-ts-checker-webpack-plugin";
import { deepMergeWithArray } from "./getUserConfig.mjs";
import eslintFormatter from "../../common/utils/eslintFormatter.mjs";
import ModuleNotFoundPlugin from "../../common/utils/ModuleNotFoundPlugin.mjs";
import InterpolateHtmlPlugin from "../../common/utils/InterpolateHtmlPlugin.mjs";
import InlineChunkHtmlPlugin from "../../common/utils/InlineChunkHtmlPlugin.mjs";
import { getEnv } from "../../common/getEnv.mjs";
const getPlugins = (userConfig, paths) => {
  const isEnvDevelopment = process.env.NODE_ENV === "development";
  const isEnvProduction = process.env.NODE_ENV === "production";
  const useSourceMap = process.env.USE_SOURCEMAP === "true";
  const useBundleAnalyzerPlugin = process.env.USE_ANALYZE === "true";
  const {
    plugin: { eslint, stylelint, typescript }
  } = userConfig;
  const env = getEnv(paths.publicUrlOrPath.slice(0, -1));
  return [
    // Generates an `index.html` file with the <script> injected.
    new HtmlWebpackPlugin({
      inject: true,
      template: paths.appHtml,
      ...isEnvProduction ? {
        minify: {
          removeComments: true,
          collapseWhitespace: true,
          removeRedundantAttributes: true,
          useShortDoctype: true,
          removeEmptyAttributes: true,
          removeStyleLinkTypeAttributes: true,
          keepClosingSlash: true,
          minifyJS: true,
          minifyCSS: true,
          minifyURLs: true
        }
      } : void 0
    }),
    // Inlines the webpack runtime script. This script is too small to warrant
    // a network request.
    // https://github.com/facebook/create-react-app/issues/5358
    isEnvProduction && new InlineChunkHtmlPlugin(HtmlWebpackPlugin, [/runtime~.+[.]js/]),
    // Makes some environment variables available in index.html.
    // The public URL is available as %PUBLIC_URL% in index.html, e.g.:
    // <link rel="icon" href="%PUBLIC_URL%/favicon.ico">
    // It will be an empty string unless you specify "homepage"
    // in `package.json`, in which case it will be the pathname of that URL.
    new InterpolateHtmlPlugin(HtmlWebpackPlugin, env.raw),
    // This gives some necessary context to module not found errors, such as
    // the requesting resource.
    new ModuleNotFoundPlugin(paths.appPath),
    // Makes some environment variables available to the JS code, for example:
    // if (process.env.NODE_ENV === 'production') { ... }. See `./env.js`.
    // It is absolutely essential that NODE_ENV is set to production
    // during a production build.
    // Otherwise React will be compiled in the very slow development mode.
    new webpack.DefinePlugin(env.stringified),
    // Experimental hot reloading for React .
    // https://github.com/facebook/react/tree/main/packages/react-refresh
    isEnvDevelopment && new ReactRefreshWebpackPlugin({
      overlay: false
    }),
    // Watcher doesn't work well if you mistype casing in a path so we use
    // a plugin that prints an error when you attempt to do this.
    // See https://github.com/facebook/create-react-app/issues/240
    isEnvDevelopment && new CaseSensitivePathsPlugin(),
    isEnvProduction && new MiniCssExtractPlugin({
      // Options similar to the same options in webpackOptions.output
      // both options are optional
      filename: "static/css/[name].[contenthash:16].css",
      chunkFilename: "static/css/[name].[contenthash:16].chunk.css"
    }),
    // Generate an asset manifest file with the following content:
    // - "files" key: Mapping of all asset filenames to their corresponding
    //   output file so that tools can pick it up without having to parse
    //   `index.html`
    // - "entrypoints" key: Array of files which are included in `index.html`,
    //   can be used to reconstruct the HTML if necessary
    isEnvProduction && new WebpackManifestPlugin({
      fileName: "asset-manifest.json",
      publicPath: paths.publicUrlOrPath,
      generate: (seed, files, entrypoints) => {
        const manifestFiles = files.reduce((manifest, file) => {
          manifest[file.name] = file.path;
          return manifest;
        }, seed);
        const entrypointFiles = entrypoints.main.filter(
          (fileName) => !fileName.endsWith(".map")
        );
        return {
          files: manifestFiles,
          entrypoints: entrypointFiles
        };
      }
    }),
    // Generate a service worker script that will precache, and keep up to date,
    // the HTML & assets that are part of the webpack build.
    isEnvProduction && fs.pathExistsSync(paths.swSrc) && new WorkboxWebpackPlugin.InjectManifest({
      swSrc: paths.swSrc,
      dontCacheBustURLsMatching: /\.[0-9a-f]{8}\./,
      exclude: [/\.map$/, /asset-manifest\.json$/, /LICENSE/],
      // Bump up the default maximum size (2mb) that's precached,
      // to make lazy-loading failure scenarios less likely.
      // See https://github.com/cra-template/pwa/issues/13#issuecomment-722667270
      maximumFileSizeToCacheInBytes: 5 * 1024 * 1024
    }),
    // TypeScript type checking
    typescript?.enable && new ForkTsCheckerWebpackPlugin(
      deepMergeWithArray(typescript?.options, {
        async: isEnvDevelopment,
        typescript: {
          typescriptPath: resolve.sync("typescript", {
            basedir: paths.appNodeModules
          }),
          configOverwrite: {
            compilerOptions: {
              sourceMap: isEnvProduction ? useSourceMap : isEnvDevelopment,
              skipLibCheck: true,
              inlineSourceMap: false,
              declarationMap: false,
              noEmit: true,
              incremental: true,
              tsBuildInfoFile: paths.appTsBuildInfoFile
            }
          },
          context: paths.appPath,
          diagnosticOptions: {
            syntactic: true
          },
          mode: "write-references"
        },
        issue: {
          // This one is specifically to match during CI tests,
          // as micromatch doesn't match
          // '../cra-template-typescript/template/src/App.tsx'
          // otherwise.
          include: [
            { file: "../**/src/**/*.{ts,tsx}" },
            { file: "**/src/**/*.{ts,tsx}" }
          ],
          exclude: [
            { file: "**/src/**/__tests__/**" },
            { file: "**/src/**/?(*.){spec|test}.*" },
            { file: "**/src/setupProxy.*" },
            { file: "**/src/setupTests.*" }
          ]
        },
        logger: "webpack-infrastructure"
      })
    ),
    eslint?.enable && new ESLintPlugin(
      deepMergeWithArray(eslint?.options, {
        extensions: ["js", "mjs", "jsx", "ts", "tsx"],
        formatter: eslintFormatter,
        eslintPath: require.resolve("eslint"),
        failOnError: true,
        context: paths.appSrc,
        cache: true,
        cacheLocation: path.resolve(
          paths.appNodeModules,
          ".cache/.eslintcache"
        ),
        cwd: paths.appPath
      })
    ),
    stylelint?.enable && new StylelintPlugin(
      deepMergeWithArray(stylelint?.options, {
        extensions: ["less", "scss", "sass", "stylus", "styl", "css"],
        stylelintPath: require.resolve("stylelint"),
        failOnError: isEnvProduction,
        context: paths.appSrc,
        cache: true,
        cacheLocation: path.resolve(
          paths.appNodeModules,
          ".cache/.stylelintcache"
        ),
        cwd: paths.appPath
      })
    ),
    useBundleAnalyzerPlugin && new BundleAnalyzerPlugin(),
    new WebpackBar()
  ].filter(Boolean);
};
export {
  getPlugins
};
