import fs from "fs-extra";
import pico from "picocolors";
import webpack from "webpack";
import {
  printFileSizesAfterBuild,
  measureFileSizesBeforeBuild
} from "../../common/utils/FileSizeReporter.mjs";
import formatWebpackMessages from "../../common/utils/formatWebpackMessages.mjs";
import printBuildError from "../../common/utils/printBuildError.mjs";
import checkRequiredFiles from "../../common/utils/checkRequiredFiles.mjs";
import { applyEnv } from "../../common/getEnv.mjs";
import { getPaths } from "../../common/getPaths.mjs";
import { getWebpackConfig } from "../config/webpack.config.mjs";
import { getUserConfig } from "../config/getUserConfig.mjs";
process.on("unhandledRejection", (err) => {
  throw err;
});
const WARN_AFTER_BUNDLE_GZIP_SIZE = 512 * 1024;
const WARN_AFTER_CHUNK_GZIP_SIZE = 1024 * 1024;
const copyPublicFolder = async (paths) => {
  await fs.copy(paths.appPublic, paths.appDist, {
    dereference: true,
    filter: (file) => file !== paths.appHtml
  });
};
const webpackBuild = async (config) => {
  console.log(pico.cyan("Creating an optimized production build..."));
  const compiler = webpack(config);
  return new Promise((resolve, reject) => {
    compiler.run((err, stats) => {
      let messages;
      if (err) {
        if (!err.message) {
          return reject(err);
        }
        let errMessage = err.message;
        if (Object.prototype.hasOwnProperty.call(err, "postcssNode")) {
          errMessage += "\nCompileError: Begins at CSS selector " + err["postcssNode"].selector;
        }
        messages = formatWebpackMessages({
          errors: [errMessage],
          warnings: []
        });
      } else {
        messages = formatWebpackMessages(
          stats?.toJson({ all: false, warnings: true, errors: true })
        );
      }
      if (messages.errors.length) {
        if (messages.errors.length > 1) {
          messages.errors.length = 1;
        }
        return reject(new Error(messages.errors.join("\n\n")));
      }
      const resolveArgs = {
        stats,
        warnings: messages.warnings
      };
      return resolve(resolveArgs);
    });
  });
};
const build = async (mode) => {
  try {
    const userConfig = await getUserConfig();
    const paths = getPaths(userConfig);
    applyEnv(mode, "production", paths);
    if (!checkRequiredFiles([paths.appHtml, paths.appIndexJs])) {
      process.exit(1);
    }
    const previousFileSizes = await measureFileSizesBeforeBuild(paths.appDist);
    await fs.emptyDir(paths.appDist);
    await copyPublicFolder(paths);
    try {
      const { stats, warnings } = await webpackBuild(
        getWebpackConfig(userConfig, paths)
      );
      if (warnings.length) {
        console.log(pico.yellow("Compiled with warnings.\n"));
        console.log(warnings.join("\n\n"));
        console.log(
          "\nSearch for the " + pico.underline(pico.yellow("keywords")) + " to learn more about each warning."
        );
        console.log(
          "To ignore, add " + pico.cyan("// eslint-disable-next-line") + " to the line before.\n"
        );
      }
      console.log("File sizes after gzip:\n");
      printFileSizesAfterBuild(
        stats,
        previousFileSizes,
        paths.appDist,
        WARN_AFTER_BUNDLE_GZIP_SIZE,
        WARN_AFTER_CHUNK_GZIP_SIZE
      );
      console.log();
    } catch (err) {
      console.log(pico.red("Failed to compile.\n"));
      printBuildError(err);
      process.exit(1);
    }
  } catch (err) {
    if (err && err.message) {
      console.log(err.message);
    }
    process.exit(1);
  }
};
var build_default = build;
export {
  build_default as default
};
